//@author: vvvv group
//@help: this is a very basic template. use it to start writing your own effects. if you want effects with lighting start from one of the GouraudXXXX or PhongXXXX effects
//@tags:
//@credits:

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWV: WORLDVIEW;
float4x4 tVP: VIEWPROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//light properties
float3 lPos <string uiname="Light Position";> = {0, 5, -2};       //light position in world space
float4 ambient  : COLOR <String uiname="Ambient Color";>  = {0.15, 0.15, 0.15, 1};
float4 diffuse  : COLOR <String uiname="Diffuse Color";>  = {0.85, 0.85, 0.85, 1};
float4 specular : COLOR <String uiname="Specular Color";> = {0.35, 0.35, 0.35, 1};
float  shiness <String uiname="Shiness"; float uimin=0.0;> = 25.0;     //shininess of specular highlight
float  specularMultiple <String uiname="Effect Multiple"; float uimin=0.0;> = 5.0;     //shininess of specular highlight
// others
int stepsPassed = 0;
float noiseStrength = 0.1;
float noiseFrequency = 1.0;
// bump properities
float  bumpMultiple <String uiname="Bump Multiple";> = 1.0;
float2  texSize <String uiname="Bump Texture Size";> = {256.0,256.0};

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
	//TAdress = clamp;
    AddressU = MIRROR;
    AddressV = MIRROR;
};
//bump map texture
texture TexBump <string uiname="Texture Bump";>;
sampler SampBump = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (TexBump);          //apply a texture to the sampler
    MipFilter = LINEAR;         	//set the sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
	//WrapS = clamp;
};

//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

float4x4 cylinderBend[10];
float cylinderLength[10];

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
	float3 Normal	: TEXCOORD1;
	float3 LightDir	: TEXCOORD2;
	float3 EyeVec	: TEXCOORD3;
};
struct vs2psBump
{
    float4 PositionClipSpace: POSITION;
    float2 TexCd : TEXCOORD0;
	float3 LightDir	: TEXCOORD1;
	float3 EyeVec	: TEXCOORD2;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    //float4 PositionClipSpace: POSITION;
    float3 NormO: NORMAL,
    float3 TangO: TANGENT,
    float4 TexCd : TEXCOORD0)
{
    //declare output struct
    vs2ps Out;

    //transform position
	PosO.y+=+.499;
	int vertexHeight = (int)((PosO.y)*10.0);
	//float3 cylinderBend[vertexHeight]
	PosO.y = (PosO.y-(vertexHeight/10.0))*(-9.5)*cylinderLength[vertexHeight];
	//PosO.y = 0;
    float4 PosCylinder = mul(PosO, cylinderBend[vertexHeight]);


    float4 PosW = mul(PosCylinder, tW);	
	//add some vertex noise
	PosW.x+= noiseStrength*sin(noiseFrequency*(PosW.z+PosW.y));
	PosW.y+= noiseStrength*sin(noiseFrequency*(PosW.x+PosW.z));
	PosW.z+= noiseStrength*sin(noiseFrequency*(PosW.y+PosW.x));
	//inverse light direction in view space
    float3 LightDirW = normalize(lPos - PosW);
    Out.LightDir = mul(LightDirW, tV);
    
    //normal in view space
	// -1* corrected to be inside Cylinder 
    Out.Normal = mul( mul(-1.0*NormO, cylinderBend[vertexHeight]), tWV);
	
    //view direction = inverse vertexposition in viewspace
    float3 PosV = mul(PosW, tV);
    Out.EyeVec = normalize(-PosV);
	
	Out.Pos = mul(PosW, tVP);
    
    //transform texturecoordinates
	//TexCd.x += stepsPassed * (1.0/20.0);
	float4 TexCd2 = TexCd;
	TexCd2.y += stepsPassed * (-1.0/10.0);
    Out.TexCd = mul(TexCd2, tTex);

    return Out;
}
vs2psBump VSBump(
    float4 PosO: POSITION,
    float3 NormO: NORMAL,
    float3 TangO: TANGENT,
    //float3 BinoO: BINORMAL,
    float4 TexCd : TEXCOORD0)
{
    vs2psBump Out = (vs2psBump)0;
	
    //transform position
	PosO.y+=+.49;
	int vertexHeight = (int)((PosO.y)*10.0);
	PosO.y = (PosO.y-(vertexHeight/10.0))*(-9.5)*cylinderLength[vertexHeight];
	//PosO.y = 0;
    float4 PosCylinder = mul(PosO, cylinderBend[vertexHeight]);

    float4 PosW = mul(PosCylinder, tW);	
	//add some vertex noise
	PosW.x+= 0.2*noiseStrength*sin(noiseFrequency*(PosW.z+PosW.y));
	PosW.y+= noiseStrength*sin(noiseFrequency*(PosW.x+PosW.z));
	PosW.z+= 0.2*noiseStrength*sin(noiseFrequency*(PosW.y+PosW.x));
	Out.PositionClipSpace  = mul(PosW, tVP);
	
    //normals -1* corrected to be inside Cylinder 
    float3 NormC = mul(-1.0*NormO, cylinderBend[vertexHeight]);
    float3 TangC = mul(-1.0*TangO, cylinderBend[vertexHeight]);
	float3 BinoO = cross(TangO,NormO);
	float3x3 TBN;
	TBN[0] = mul(TangO, tW);
	TBN[1] = mul(BinoO, tW);
	TBN[2] = mul(NormO, tW);
	TBN = transpose(TBN);	// this means multiplying from right
	
    	// vert2light direction in tangent space
    float3 LightDirW = (lPos - PosW);	// light pos - vertex pos in world space
    Out.LightDir = mul(LightDirW,TBN);

    	// vert2camera direction = inverse vertexposition in screen, to tangent space
    float3 PosV = mul(PosW, tV);
    Out.EyeVec = mul(float3(-PosV), TBN);
	
    //transform texturecoordinates
	//TexCd.x += stepsPassed * (1.0/20.0);
	float4 TexCd2 = TexCd;
	TexCd2.y += stepsPassed * (-1.0/10.0);
    Out.TexCd = mul(TexCd2, tTex);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
    float4 textureDiff = tex2D(Samp, In.TexCd);
	
		//	ambient
	float4 final_color = 0;
	final_color += ambient*textureDiff;	// can be multiplied by texture
	
		//	diffuse
	//float3 N = bumpNormal;
	float3 N = normalize(In.Normal);
	float3 L = normalize(In.LightDir);
	float lambertTerm = max(dot(N,L), 0.0);
	final_color.rgb += diffuse.rgb * textureDiff.rgb * lambertTerm;
	
		//	specular
	float3 E = normalize(In.EyeVec);
	float3 R = reflect(-L, N);
	float specularFloat = specularMultiple*pow( max(dot(R,E), 0.0), shiness );
	final_color.rgb += specular.rgb * textureDiff.rgb * textureDiff.rgb * specularFloat;

    return final_color;
}
float4 PSBump(vs2psBump In): COLOR
{
	// fragment normals are counted from a height map
    float4 textureDiff = tex2D(Samp, In.TexCd);
    float4 textureBumpUp = tex2D(SampBump, float2(In.TexCd.x,In.TexCd.y-1.0/texSize.y));
    float4 textureBumpDo = tex2D(SampBump, float2(In.TexCd.x,In.TexCd.y+1.0/texSize.y));
    float4 textureBumpLe = tex2D(SampBump, float2(In.TexCd.x-1.0/texSize.x,In.TexCd.y));
    float4 textureBumpRi = tex2D(SampBump, float2(In.TexCd.x+1.0/texSize.x,In.TexCd.y));
	// only red color makes the grey
	float3 tangentT = float3( 0, 1, bumpMultiple*(textureBumpUp.r-textureBumpDo.r));
	float3 tangentS = float3( 1, 0, bumpMultiple*(textureBumpRi.r-textureBumpLe.r));
	float3 bumpNormal = normalize(cross(tangentS,tangentT));
	
	//float3 bumpNormal = (2.0 * (tex2D(BumpMapSampler,input.TexCoord))) - 1.0;
	
		//	ambient
	float4 final_color = 0;
	final_color += ambient*textureDiff;	// can be multiplied by texture
	
		//	diffuse
	float3 N = bumpNormal;
	//N = float3(0,0,1);
	float3 L = normalize(In.LightDir);
	float lambertTerm = max(dot(N,L), 0.0);
	final_color.rgb += textureDiff.rgb*diffuse.rgb*lambertTerm;
	
		//	specular
	float3 E = normalize(In.EyeVec);
	float3 R = reflect(-L, N);
	float specularFloat = specularMultiple*pow( max(dot(R,E), 0.0), shiness );
	final_color.rgb += textureDiff.rgb*specular.rgb * specularFloat;

    return final_color;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique TPhongTunel
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS();
    	//fillmode = wireframe;
    }
}
technique TBumpTunel
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_3_0 VSBump();
        PixelShader  = compile ps_3_0 PSBump();
    	//fillmode = wireframe;
    }
}
technique TWireframeTunel
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS();
    	fillmode = wireframe;
    }
}

